import 'dart:async';

import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:neon/src/models/account.dart';
import 'package:neon/src/sync/models/mapping.dart';
import 'package:synchronize/synchronize.dart';

@immutable
abstract interface class SyncImplementation<S extends SyncMapping<T1, T2>, T1, T2> {
  String get appId;

  FutureOr<SyncSources<T1, T2>> getSources(final Account account, final S mapping);

  Map<String, dynamic> serializeMapping(final S mapping);

  S deserializeMapping(final Map<String, dynamic> json);

  FutureOr<S?> addMapping(final BuildContext context, final Account account);

  String getMappingDisplayTitle(final S mapping);

  String getMappingDisplaySubtitle(final S mapping);

  String getMappingId(final S mapping);

  Widget getConflictDetailsLocal(final BuildContext context, final T2 object);

  Widget getConflictDetailsRemote(final BuildContext context, final T1 object);
}

extension SyncImplementationGlobalUniqueMappingId
    on SyncImplementation<SyncMapping<dynamic, dynamic>, dynamic, dynamic> {
  String getGlobalUniqueMappingId(final SyncMapping<dynamic, dynamic> mapping) =>
      '${mapping.accountId}-${mapping.appId}-${getMappingId(mapping)}';
}

extension SyncImplementationFind on Iterable<SyncImplementation<SyncMapping<dynamic, dynamic>, dynamic, dynamic>> {
  SyncImplementation<SyncMapping<dynamic, dynamic>, dynamic, dynamic>? tryFind(final String appId) =>
      singleWhereOrNull((final syncImplementation) => appId == syncImplementation.appId);

  SyncImplementation<SyncMapping<dynamic, dynamic>, dynamic, dynamic> find(final String appId) => tryFind(appId)!;
}
